#include <iostream>
#include <string>
#include <time.h>
#include "thmath.h"
using namespace std;

int main (int argc, char * const argv[]) {
	string op, a, b, s;
    if (argc == 1)
	{
		cout << "Usage: thmath operation [argument1 [argument2]]" << endl;
		return 0;
	}
	op = argv[1];
	if (argc > 2) { a = argv[2]; }
	if (argc > 3) { b = argv[3]; }
	if (op == (string)"listops")
	{
		cout << "Operations supported:" << endl;
		cout << "   Testing:" << endl;
		cout << "      listops       test            signtest a b    signtest2 a b" << endl;
		cout << "   Arithmetic:" << endl;
		cout << "      zero a        equal a b       compare a b     trim a" << endl;
		cout << "      add a b       subtract a b    multiply a b    divide a b" << endl;
		cout << "      div a b       mod a b         pow a b" << endl;
		cout << "      trunc a       abs a           square a        sqrt a" << endl;
		cout << "      sign a" << endl;
		cout << "   Logarithms (WARNING! SLOW!):" << endl;
		cout << "      ln a          exp a" << endl;
		cout << "   Trigonometry (WARNING! SLOW!):" << endl;
		cout << "      sin a         cos a           tan a" << endl;
		cout << "      cot a         sec a           csc a" << endl;
		cout << "      asin a        acos a          atan a" << endl;
		cout << "      acot a        asec a          acsc a" << endl;
		cout << "   Hyperbolic (WARNING! SLOW!):" << endl;
		cout << "      sinh a        cosh a          tanh a" << endl;
		cout << "      coth a        sech a          csch a" << endl;
		cout << "      asinh a       acosh a         atanh a" << endl;
		cout << "      acoth a       asech a         acsch a" << endl;
		cout << "   Gamma (SLOW for non-multiples-of-1/2):" << endl;
		cout << "      factorial a   gamma a" << endl;
		cout << "   Complex Numbers:" << endl;
		cout << "      czero a       cequal a b      ccompare a b    ctrim a" << endl;
		cout << "      cadd a b      csubtract a b   cmultiply a b   cdivide a b" << endl;
		cout << "      cdiv a b      cmod a b" << endl;
		cout << "      ctrunc a      cabs a          csquare a       csqrt a" << endl;
		cout << "      csign a" << endl;
		return 0;
	}
	if (op == (string)"test")
	{
		clock_t tstart, tend;
		long smops;
		int reps = 100000;
		tstart = clock();
		th__test(20, 0, reps);
		tend = clock();
		smops = ((reps * CLK_TCK) / (tend - tstart));
		cout << "Addition: " << smops << " smops @ 20 digits" << endl;
		tstart = clock();
		th__test(20, 1, reps);
		tend = clock();
		smops = ((reps * CLK_TCK) / (tend - tstart));
		cout << "Subtraction: " << smops << " smops @ 20 digits" << endl;
		reps = 10000;
		tstart = clock();
		th__test(20, 2, reps);
		tend = clock();
		smops = ((reps * CLK_TCK) / (tend - tstart));
		cout << "Multiplication: " << smops << " smops @ 20 digits" << endl;
		reps = 100;
		tstart = clock();
		th__test(20, 3, reps);
		tend = clock();
		smops = ((reps * CLK_TCK) / (tend - tstart));
		cout << "Division: " << smops << " smops @ 20 digits" << endl;
		tstart = clock();
		th__test(20, 4, reps);
		tend = clock();
		smops = ((reps * CLK_TCK) / (tend - tstart));
		cout << "Integer Power: " << smops << " smops @ 20 digits" << endl;
		tstart = clock();
		th__test(20, 5, reps);
		tend = clock();
		smops = ((reps * CLK_TCK) / (tend - tstart));
		cout << "Sqrt: " << smops << " smops @ 20 digits" << endl;
		tstart = clock();
		return 0;
	}
	if (op == (string)"test2")
	{
		clock_t tstart, tend;
		long smops;
		int reps = 100000;
		tstart = clock();
		th__test(10, 0, reps);
		tend = clock();
		smops = ((reps * CLK_TCK) / (tend - tstart));
		cout << "Addition: " << smops << " smops @ 10 digits" << endl;
		tstart = clock();
		th__test(10, 1, reps);
		tend = clock();
		smops = ((reps * CLK_TCK) / (tend - tstart));
		cout << "Subtraction: " << smops << " smops @ 10 digits" << endl;
		reps = 10000;
		tstart = clock();
		th__test(10, 2, reps);
		tend = clock();
		smops = ((reps * CLK_TCK) / (tend - tstart));
		cout << "Multiplication: " << smops << " smops @ 10 digits" << endl;
		reps = 100;
		tstart = clock();
		th__test(10, 3, reps);
		tend = clock();
		smops = ((reps * CLK_TCK) / (tend - tstart));
		cout << "Division: " << smops << " smops @ 10 digits" << endl;
		tstart = clock();
		th__test(10, 4, reps);
		tend = clock();
		smops = ((reps * CLK_TCK) / (tend - tstart));
		cout << "Integer Power: " << smops << " smops @ 10 digits" << endl;
		tstart = clock();
		th__test(10, 5, reps);
		tend = clock();
		smops = ((reps * CLK_TCK) / (tend - tstart));
		cout << "Sqrt: " << smops << " smops @ 10 digits" << endl;
		tstart = clock();
		return 0;
	}
	if (op == (string)"signtest")
	{
		if (argc < 4) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = (string)" "+a+(string)" +  "+b+(string)" = " + th_add(a,b);
		cout << s.c_str() << endl;
		s = (string)" "+a+(string)" + -"+b+(string)" = " + th_add(a,(string)"-" + b);
		cout << s.c_str() << endl;
		s = (string)"-"+a+(string)" + -"+b+(string)" = " + th_add((string)"-" + a,(string)"-" + b);
		cout << s.c_str() << endl;
		s = (string)"-"+a+(string)" +  "+b+(string)" = " + th_add((string)"-" + a,b);
		cout << s.c_str() << endl;
		s = (string)" "+b+(string)" +  "+a+(string)" = " + th_add(b,a);
		cout << s.c_str() << endl;
		s = (string)" "+b+(string)" + -"+a+(string)" = " + th_add(b,(string)"-" + a);
		cout << s.c_str() << endl;
		s = (string)"-"+b+(string)" + -"+a+(string)" = " + th_add((string)"-" + b,(string)"-" + a);
		cout << s.c_str() << endl;
		s = (string)"-"+b+(string)" +  "+a+(string)" = " + th_add((string)"-" + b,a);
		cout << s.c_str() << endl;
		s = (string)" "+a+(string)" -  "+b+(string)" = " + th_subtract(a,b);
		cout << s.c_str() << endl;
		s = (string)" "+a+(string)" - -"+b+(string)" = " + th_subtract(a,(string)"-" + b);
		cout << s.c_str() << endl;
		s = (string)"-"+a+(string)" - -"+b+(string)" = " + th_subtract((string)"-" + a,(string)"-" + b);
		cout << s.c_str() << endl;
		s = (string)"-"+a+(string)" -  "+b+(string)" = " + th_subtract((string)"-" + a,b);
		cout << s.c_str() << endl;
		s = (string)" "+b+(string)" -  "+a+(string)" = " + th_subtract(b,a);
		cout << s.c_str() << endl;
		s = (string)" "+b+(string)" - -"+a+(string)" = " + th_subtract(b,(string)"-" + a);
		cout << s.c_str() << endl;
		s = (string)"-"+b+(string)" - -"+a+(string)" = " + th_subtract((string)"-" + b,(string)"-" + a);
		cout << s.c_str() << endl;
		s = (string)"-"+b+(string)" -  "+a+(string)" = " + th_subtract((string)"-" + b,a);
		cout << s.c_str() << endl;
		s = (string)" "+a+(string)" *  "+b+(string)" = " + th_multiply(a,b);
		cout << s.c_str() << endl;
		s = (string)" "+a+(string)" * -"+b+(string)" = " + th_multiply(a,(string)"-" + b);
		cout << s.c_str() << endl;
		s = (string)"-"+a+(string)" * -"+b+(string)" = " + th_multiply((string)"-" + a,(string)"-" + b);
		cout << s.c_str() << endl;
		s = (string)"-"+a+(string)" *  "+b+(string)" = " + th_multiply((string)"-" + a,b);
		cout << s.c_str() << endl;
		s = (string)" "+b+(string)" *  "+a+(string)" = " + th_multiply(b,a);
		cout << s.c_str() << endl;
		s = (string)" "+b+(string)" * -"+a+(string)" = " + th_multiply(b,(string)"-" + a);
		cout << s.c_str() << endl;
		s = (string)"-"+b+(string)" * -"+a+(string)" = " + th_multiply((string)"-" + b,(string)"-" + a);
		cout << s.c_str() << endl;
		s = (string)"-"+b+(string)" *  "+a+(string)" = " + th_multiply((string)"-" + b,a);
		cout << s.c_str() << endl;
		s = (string)" "+a+(string)" /  "+b+(string)" = " + th_divide(a,b);
		cout << s.c_str() << endl;
		s = (string)" "+a+(string)" / -"+b+(string)" = " + th_divide(a,(string)"-" + b);
		cout << s.c_str() << endl;
		s = (string)"-"+a+(string)" / -"+b+(string)" = " + th_divide((string)"-" + a,(string)"-" + b);
		cout << s.c_str() << endl;
		s = (string)"-"+a+(string)" /  "+b+(string)" = " + th_divide((string)"-" + a,b);
		cout << s.c_str() << endl;
		s = (string)" "+b+(string)" /  "+a+(string)" = " + th_divide(b,a);
		cout << s.c_str() << endl;
		s = (string)" "+b+(string)" / -"+a+(string)" = " + th_divide(b,(string)"-" + a);
		cout << s.c_str() << endl;
		s = (string)"-"+b+(string)" / -"+a+(string)" = " + th_divide((string)"-" + b,(string)"-" + a);
		cout << s.c_str() << endl;
		s = (string)"-"+b+(string)" /  "+a+(string)" = " + th_divide((string)"-" + b,a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"signtest2")
	{
		if (argc < 4) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = (string)" "+a+(string)" =  "+b+(string)" = " + th_compare(a,b);
		cout << s.c_str() << endl;
		s = (string)" "+a+(string)" = -"+b+(string)" = " + th_compare(a,(string)"-" + b);
		cout << s.c_str() << endl;
		s = (string)"-"+a+(string)" = -"+b+(string)" = " + th_compare((string)"-" + a,(string)"-" + b);
		cout << s.c_str() << endl;
		s = (string)"-"+a+(string)" =  "+b+(string)" = " + th_compare((string)"-" + a,b);
		cout << s.c_str() << endl;
		s = (string)" "+b+(string)" =  "+a+(string)" = " + th_compare(b,a);
		cout << s.c_str() << endl;
		s = (string)" "+b+(string)" = -"+a+(string)" = " + th_compare(b,(string)"-" + a);
		cout << s.c_str() << endl;
		s = (string)"-"+b+(string)" = -"+a+(string)" = " + th_compare((string)"-" + b,(string)"-" + a);
		cout << s.c_str() << endl;
		s = (string)"-"+b+(string)" =  "+a+(string)" = " + th_compare((string)"-" + b,a);
		cout << s.c_str() << endl;
		s = (string)" "+a+(string)" ^  "+b+(string)" = " + th_pow(a,b);
		cout << s.c_str() << endl;
		s = (string)" "+a+(string)" ^ -"+b+(string)" = " + th_pow(a,(string)"-" + b);
		cout << s.c_str() << endl;
		s = (string)"-"+a+(string)" ^ -"+b+(string)" = " + th_pow((string)"-" + a,(string)"-" + b);
		cout << s.c_str() << endl;
		s = (string)"-"+a+(string)" ^  "+b+(string)" = " + th_pow((string)"-" + a,b);
		cout << s.c_str() << endl;
		s = (string)" "+b+(string)" ^  "+a+(string)" = " + th_pow(b,a);
		cout << s.c_str() << endl;
		s = (string)" "+b+(string)" ^ -"+a+(string)" = " + th_pow(b,(string)"-" + a);
		cout << s.c_str() << endl;
		s = (string)"-"+b+(string)" ^ -"+a+(string)" = " + th_pow((string)"-" + b,(string)"-" + a);
		cout << s.c_str() << endl;
		s = (string)"-"+b+(string)" ^  "+a+(string)" = " + th_pow((string)"-" + b,a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"equal")
	{
		if (argc < 4) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_equal(a,b)?(string)"true":(string)"false";
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"compare")
	{
		if (argc < 4) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_compare(a,b);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"add")
	{
		if (argc < 4) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_add(a,b);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"subtract")
	{
		if (argc < 4) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_subtract(a,b);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"multiply")
	{
		if (argc < 4) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_multiply(a,b);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"divide")
	{
		if (argc < 4) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_divide(a,b);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"div")
	{
		if (argc < 4) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_div(a,b);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"mod")
	{
		if (argc < 4) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_mod(a,b);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"pow")
	{
		if (argc < 4) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_pow(a,b);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"cequal")
	{
		if (argc < 4) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_cequal(a,b)?(string)"true":(string)"false";
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"ccompare")
	{
		if (argc < 4) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_ccompare(a,b);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"cadd")
	{
		if (argc < 4) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_cadd(a,b);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"csubtract")
	{
		if (argc < 4) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_csubtract(a,b);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"cmultiply")
	{
		if (argc < 4) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_cmultiply(a,b);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"cdivide")
	{
		if (argc < 4) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_cdivide(a,b);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"cdiv")
	{
		if (argc < 4) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_cdiv(a,b);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"cmod")
	{
		if (argc < 4) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_cmod(a,b);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"cpow")
	{
		if (argc < 4) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_cpow(a,b);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"zero")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_zero(a)?(string)"true":(string)"false";
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"czero")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_czero(a)?(string)"true":(string)"false";
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"trim")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_trim(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"trunc")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_trunc(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"sqrt")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_sqrt(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"square")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_square(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"abs")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_abs(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"sign")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_sign(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"ctrim")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_ctrim(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"ctrunc")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_ctrunc(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"csqrt")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_csqrt(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"csquare")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_csquare(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"cabs")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_cabs(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"csign")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_csign(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"gamma")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_gamma(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"factorial")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_factorial(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"ln")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_ln(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"exp")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_exp(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"sin")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_sin(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"cos")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_cos(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"tan")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_tan(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"cot")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_cot(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"sec")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_sec(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"csc")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_csc(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"atan")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_atan(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"sinh")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_sinh(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"cosh")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_cosh(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"tanh")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_tanh(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"coth")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_coth(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"sech")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_sech(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"csch")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_csch(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"atanh")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_atanh(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"asin")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_asin(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"acos")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_acos(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"acot")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_acot(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"asec")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_asec(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"acsc")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_acsc(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"asinh")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_asinh(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"acosh")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_acosh(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"acoth")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_acoth(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"asech")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_asech(a);
		cout << s.c_str() << endl;
		return 0;
	}
	if (op == (string)"acsch")
	{
		if (argc < 3) { cout << "Not enough parameters given to operation." << endl; return 0; }
		s = th_acsch(a);
		cout << s.c_str() << endl;
		return 0;
	}
	cout << "Unknown operation." << endl;
    return 0;
}
