/*

 Thmath
 It'th a thmath to uthe thring math!
 (c) 2005 Jonathan Bettencourt / Kreative Korporation

 Thith here thee++ code ith a port of ThtringMathLib in RealBathic
 and MathLib in HyperCard. It will do thimple thtring math on thtrings
 of arbitrary length, with no limit to prethithion.


 Addition, subtraction, multiplication, and positive integer powers
 are *unlimited* in precision, regardless of the setting of th_precision.

 Division, modulus, square root, and powers that are multiples of 1/2
 are limited to the precision set by th_precision.

 Trigonometric functions, hyperbolic functions, logarithmic functions,
 gamma functions, and real number powers are limited to a precision of
 either th_precision or a little more than 500, whichever is less.

 To increase (or decrease) the precision of trig, hyp, log, gamma functions:
	* Add more digits to the constants in th_constants.cpp
	* Increase the value of th_const_prec in thmath.cpp
	* Add more factorials to th_factorialtable.cpp
	* Increase the value of th_max_factorial in thmath.cpp


 This code is under the MIT license.

 Permission is hereby granted, free of charge, to any person obtaining a copy of
 this software and associated documentation files (the "Software"), to deal in the
 Software without restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 Software, and to permit persons to whom the Software is furnished to do so,
 subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies
 or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/

#include <string>
#include "th_factorialtable.h"
#include "th_constants.h"
#include "thmath.h"
using namespace std;

int th_precision = 100;				/* number of digits after the decimal point */
string th_lastquotient = (string)"0";		/* storage area for faster division */

const int th_const_prec = 510;			/* number of digits after the decimal point in the constants in th_constants.cpp */
const int th_maclaur_limit = 2147483647;	/* one more than the highest term of the Maclaurin series of ln / atan / atanh */
const int th_max_factorial = 256;		/* one more than the highest factorial in th_factorialtable.cpp */

int th_gprecision(void)
{
	return th_precision;
}

void th_sprecision(int x)
{
	th_precision = x;
}

bool th_zero(string a)
{
	string t = th_trim(a);
	return (t == (string)"0");
}

bool th_equal(string a, string b)
{
	string aa = a;
	string bb = b;
	string signa;
	string signb;
	int c;
	if (aa[0] == '-') { signa = (string)"-"; aa.erase(0,1); }
	if (bb[0] == '-') { signb = (string)"-"; bb.erase(0,1); }
	if (aa[0] == '+') { aa.erase(0,1); }
	if (bb[0] == '+') { bb.erase(0,1); }

	if (th_zero(aa) && th_zero(bb)) { return true; }
	else if (signa != signb) { return false; }
	else if ( (aa == (string)"INF") && (bb == (string)"INF") ) { return true; }
	else if ( (aa == (string)"INF") || (bb == (string)"INF") ) { return false; }
	else if ( (aa == (string)"NAN") || (bb == (string)"NAN") ) { return false; }
	aa = th_trim(aa);
	bb = th_trim(bb);
	if (aa.size() != bb.size()) { return false; }
	else
	{
		for (c = 0; c < (int)aa.size(); c++)
		{
			if (aa[c] == ',') { aa[c] = '.'; }
			if (bb[c] == ',') { bb[c] = '.'; }
			if (aa[c] != bb[c]) { return false; }
		}
	}
	return true;
}

string th_compare(string a, string b)
{
	string aa = a;
	string bb = b;
	string signa;
	string signb;
	int o1, o2, nz;
	int c;
	string z;
	if (aa[0] == '-') { signa = (string)"-"; aa.erase(0,1); }
	if (bb[0] == '-') { signb = (string)"-"; bb.erase(0,1); }
	if (aa[0] == '+') { aa.erase(0,1); }
	if (bb[0] == '+') { bb.erase(0,1); }

	if ( (aa == (string)"NAN") || (bb == (string)"NAN") ) { return (string)"NAN"; }
	else if (signa == (string)"-") { if (signb == (string)"-") { return th_compare(bb,aa); } else { return (string)"-1"; } }
	else if (signb == (string)"-") { return (string)"1"; }
	else
	{
		if (aa == (string)"INF") { if (bb == (string)"INF") { return "0"; } else { return "1"; } }
		else if (bb == (string)"INF") { return "-1"; }

		if ( (int)aa.find('.')>=0 || (int)aa.find(',')>=0 || (int)bb.find('.')>=0 || (int)bb.find(',')>=0 )
		{
			aa.insert(0, (string)"0");
			bb.insert(0, (string)"0");
			o1 = (int)aa.find('.');
			if (o1 < 0) { o1 = (int)aa.find(','); }
			o2 = (int)bb.find('.');
			if (o2 < 0) { o2 = (int)bb.find(','); }
			if (o1 >= 0)
			{
				aa.erase(o1,1);
				o1--;
			}
			if (o2 >= 0)
			{
				bb.erase(o2,1);
				o2--;
			}
			if (o1 >= 0) { o1 = (int)aa.size() - o1 - 1; } else { o1 = 0; }
			if (o2 >= 0) { o2 = (int)bb.size() - o2 - 1; } else { o2 = 0; }
			nz = th__diff(o1,o2);
			if (nz > 0)
			{
				z = (string)"0";
				while ((int)z.size() < nz) { z += z; }
				z = z.substr(0, nz);
				if (o1<o2) { aa += z; }
				else { bb += z; }
			}
		}
		aa = th_trim(aa);
		bb = th_trim(bb);
		if ( (int)aa.size() < (int)bb.size() ) { return (string)"-1"; }
		else if ( (int)aa.size() > (int)bb.size() ) { return (string)"1"; }
		else
		{
			for (c = 0; c < aa.size(); c++)
			{
				if (aa[c] < bb[c]) { return (string)"-1"; }
				else if (aa[c] > bb[c]) { return (string)"1"; }
			}
			return (string)"0";
		}
	}
}

string th_add(string a, string b)
{
	string aa = a;
	string bb = b;
	string sign;
	string signa;
	string signb;
	int o1, o2, nz;
	int c, q;
	string z;
	string zz;
	string cmpRslt;
	string t;
	string dgt = (string)" ";
	int carry;
	if (aa[0] == '-') { signa = (string)"-"; aa.erase(0,1); }
	if (bb[0] == '-') { signb = (string)"-"; bb.erase(0,1); }
	if (aa[0] == '+') { aa.erase(0,1); }
	if (bb[0] == '+') { bb.erase(0,1); }

	if (th_zero(aa)) { if (th_zero(bb)) { return (string)"0"; } else { return signb + th_trim(bb); } }
	else if (th_zero(bb)) { return signa + th_trim(aa); }
	else if ( (aa == (string)"NAN") || (bb == (string)"NAN") ) { return (string)"NAN"; }
	else if (aa == (string)"INF")
	{
		if ( (bb == (string)"INF") && (signa != signb) ) { return (string)"NAN"; }
		else if ( (bb == (string)"INF") && (signa == (string)"-") ) { return (string)"-INF"; }
		else { return signa + (string)"INF"; }
	}
	else if (bb == (string)"INF") { return signb + (string)"INF"; }

	if ( (int)aa.find('.')>=0 || (int)aa.find(',')>=0 || (int)bb.find('.')>=0 || (int)bb.find(',')>=0 )
	{
		aa.insert(0, (string)"0");
		bb.insert(0, (string)"0");
		o1 = (int)aa.find('.');
		if (o1 < 0) { o1 = (int)aa.find(','); }
		o2 = (int)bb.find('.');
		if (o2 < 0) { o2 = (int)bb.find(','); }
		if (o1 >= 0)
		{
			aa.erase(o1, 1);
			o1--;
		}
		if (o2 >= 0)
		{
			bb.erase(o2, 1);
			o2--;
		}
		if (o1 >= 0) { o1 = (int)aa.size() - o1 - 1; } else { o1 = 0; }
		if (o2 >= 0) { o2 = (int)bb.size() - o2 - 1; } else { o2 = 0; }
		nz = th__diff(o1,o2);
		if (nz > 0)
		{
			z = (string)"0";
			while ((int)z.size() < nz) { z += z; }
			z = z.substr(0, nz);
			if (o1<o2) { aa += z; }
			else { bb += z; }
		}
		nz += th__min(o1,o2);
	}
	else { nz = 0; }

	if (signa == signb) { sign = signa; }
	else
	{
		cmpRslt = th_compare(aa,bb);
		if ( cmpRslt == (string)"1" ) { t = signa + th_subtract(aa,bb); }
		else if ( cmpRslt == (string)"-1" ) { t = signb + th_subtract(bb,aa); }
		else { return (string)"0"; }

		if (nz > 0)
		{
			q = (int)t.size() - nz;
			if (q < 0)
			{
				q *= -1;
				zz = (string)"0";
				while ((int)zz.size() < q) { zz += zz; }
				t.insert(0, zz.substr(0,q));
				t.insert(0, (string)".");
			}
			else
			{
				t.insert(q, (string)".");
			}
		}

		return th_trim(t);
	}

	zz = (string)"0";
	q = th__diff( (int)aa.size(), (int)bb.size() );
	while ((int)zz.size() < q) { zz += zz; }
	zz = zz.substr(0, q);
	if ( (int)bb.size() < (int)aa.size() ) { bb.insert(0, zz); }
	else if ( (int)aa.size() < (int)bb.size() ) { aa.insert(0, zz); }

	carry = 0;
	for (c = (int)bb.size()-1; c >= 0; c--)
	{
		q = aa[c] + bb[c] + carry - 96;
		aa[c] = (q % 10) + 48;
		carry = q / 10;
	}
	if (carry > 0) { dgt[0] = carry + 48; aa.insert(0, dgt); }

	if (nz > 0)
	{
		q = (int)aa.size() - nz;
		if (q < 0)
		{
			q *= -1;
			t = (string)"0";
			while ((int)t.size() < q) { t += t; }
			aa.insert(0, t.substr(0,q));
			aa.insert(0, (string)".");
		}
		else
		{
			aa.insert(q, (string)".");
		}
	}

	return sign + th_trim(aa);
}

string th_subtract(string a, string b)
{
	string aa = a;
	string bb = b;
	string sign;
	string signa;
	string signb;
	int o1, o2, nz;
	int c, d, q;
	string z;
	string zz;
	string cmpRslt;
	string t;
	int carry;
	if (aa[0] == '-') { signa = (string)"-"; aa.erase(0,1); }
	if (bb[0] == '-') { signb = (string)"-"; bb.erase(0,1); }
	if (aa[0] == '+') { aa.erase(0,1); }
	if (bb[0] == '+') { bb.erase(0,1); }

	if (th_zero(aa))
	{
		if (th_zero(bb)) { return (string)"0"; }
		else if (signb == (string)"") { return (string)"-" + th_trim(bb); }
		else if (signb == (string)"-") { return th_trim(bb); }
	}
	else if (th_zero(bb)) { return signa + th_trim(aa); }
	else if ( (aa == (string)"NAN") || (bb == (string)"NAN") ) { return (string)"NAN"; }
	else if (aa == (string)"INF")
	{
		if ( (bb == (string)"INF") && (signa == signb) ) { return (string)"NAN"; }
		else if ( (bb == (string)"INF") && (signa == (string)"-") ) { return (string)"-INF"; }
		else { return signa + (string)"INF"; }
	}
	else if (bb == (string)"INF")
	{
		if (signb == (string)"") { return (string)"-INF"; }
		else { return (string)"INF"; }
	}

	if ( (int)aa.find('.')>=0 || (int)aa.find(',')>=0 || (int)bb.find('.')>=0 || (int)bb.find(',')>=0 )
	{
		aa.insert(0, (string)"0");
		bb.insert(0, (string)"0");
		o1 = (int)aa.find('.');
		if (o1 < 0) { o1 = (int)aa.find(','); }
		o2 = (int)bb.find('.');
		if (o2 < 0) { o2 = (int)bb.find(','); }
		if (o1 >= 0)
		{
			aa.erase(o1, 1);
			o1--;
		}
		if (o2 >= 0)
		{
			bb.erase(o2, 1);
			o2--;
		}
		if (o1 >= 0) { o1 = (int)aa.size() - o1 - 1; } else { o1 = 0; }
		if (o2 >= 0) { o2 = (int)bb.size() - o2 - 1; } else { o2 = 0; }
		nz = th__diff(o1,o2);
		if (nz > 0)
		{
			z = (string)"0";
			while ((int)z.size() < nz) { z += z; }
			z = z.substr(0, nz);
			if (o1<o2) { aa += z; }
			else { bb += z; }
		}
		nz += th__min(o1,o2);
	}
	else { nz = 0; }

	if (signa == signb) { sign = signa; }
	else
	{
		t = signa + th_add(aa,bb);
		if (nz > 0)
		{
			q = (int)t.size() - nz;
			if (q < 0)
			{
				q *= -1;
				zz = (string)"0";
				while ((int)zz.size() < q) { zz += zz; }
				t.insert(0, zz.substr(0,q));
				t.insert(0, (string)".");
			}
			else
			{
				t.insert(q, (string)".");
			}
		}
		return th_trim(t);
	}

	cmpRslt = th_compare(aa,bb);
	if ( cmpRslt == (string)"-1" )
	{
		t = bb;
		bb = aa;
		aa = t;
		if (sign == (string)"") { sign = (string)"-"; }
		else { sign = (string)""; }
	}

	zz = (string)"0";
	q = th__diff( (int)aa.size(), (int)bb.size() );
	while ((int)zz.size() < q) { zz += zz; }
	zz = zz.substr(0, q);
	if ( (int)bb.size() < (int)aa.size() ) { bb.insert(0, zz); }
	else if ( (int)aa.size() < (int)bb.size() ) { aa.insert(0, zz); }

	for (c = bb.size()-1; c >= 0; c--)
	{
		carry = 0;
		if (bb[c] > aa[c])
		{
			for (d = c-1; d >= 0; d--)
			{
				if (aa[d] == '0') { aa[d]='9'; }
				else
				{
					aa[d]--;
					carry = 10;
					break;
				}
			}
		}
		aa[c] = (aa[c]+48+carry-bb[c]);
	}

	if (nz > 0)
	{
		q = (int)aa.size() - nz;
		if (q < 0)
		{
			q *= -1;
			t = (string)"0";
			while ((int)t.size() < q) { t += t; }
			aa.insert(0, t.substr(0,q));
			aa.insert(0, (string)".");
		}
		else
		{
			aa.insert(q, (string)".");
		}
	}

	return sign + th_trim(aa);
}

string th_multiply(string a, string b)
{
	string aa = a;
	string bb = b;
	string sign;
	string signa;
	string signb;
	int o1, o2, nz;
	int c, d, q;
	string t;
	int carry;
	string dgt = (string)" ";
	string placeShift;
	string tcum;
	string product;
	if (aa[0] == '-') { signa = (string)"-"; aa.erase(0,1); }
	if (bb[0] == '-') { signb = (string)"-"; bb.erase(0,1); }
	if (aa[0] == '+') { aa.erase(0,1); }
	if (bb[0] == '+') { bb.erase(0,1); }

	if ( (int)aa.find('.')>=0 || (int)aa.find(',')>=0 || (int)bb.find('.')>=0 || (int)bb.find(',')>=0 )
	{
		aa = (string)"0" + th_trim(aa);
		bb = (string)"0" + th_trim(bb);
		o1 = (int)aa.find('.');
		if (o1 < 0) { o1 = (int)aa.find(','); }
		o2 = (int)bb.find('.');
		if (o2 < 0) { o2 = (int)bb.find(','); }
		if (o1 >= 0)
		{
			aa.erase(o1, 1);
			o1--;
		}
		if (o2 >= 0)
		{
			bb.erase(o2, 1);
			o2--;
		}
		if (o1 >= 0) { o1 = (int)aa.size() - o1 - 1; } else { o1 = 0; }
		if (o2 >= 0) { o2 = (int)bb.size() - o2 - 1; } else { o2 = 0; }
		nz = o1 + o2;
	}
	else { nz = 0; }

	aa = th_trim(aa);
	bb = th_trim(bb);

	if ( signa == signb ) { sign = (string)""; }
	else { sign = (string)"-"; }

	if ( (aa == (string)"NAN") || (bb == (string)"NAN") ) { return (string)"NAN"; }
	else if ( th_zero(aa) && th_zero(bb) ) { return (string)"0"; }
	else if ( (aa == (string)"INF") && th_zero(bb) ) { return (string)"NAN"; }
	else if ( (bb == (string)"INF") && th_zero(aa) ) { return (string)"NAN"; }
	else if ( (aa == (string)"INF") || (bb == (string)"INF") ) { return sign + (string)"INF"; }
	else if ( th_zero(aa) || th_zero(bb) ) { return (string)"0"; }
	else if ( th_equal(a, (string)"1") ) { return b; }
	else if ( th_equal(b, (string)"1") ) { return a; }
	else if ( th_equal(a, (string)"-1") ) { return th_subtract((string)"0",b); }
	else if ( th_equal(b, (string)"-1") ) { return th_subtract((string)"0",a); }
	else if ( th_equal(a, (string)"2") ) { return th_add(b,b); }
	else if ( th_equal(b, (string)"2") ) { return th_add(a,a); }

	carry = 0;
	placeShift = (string)"";
	product = (string)"0";
	for (c = (int)bb.size()-1; c >= 0; c--)
	{
		tcum = (string)"";
		for (d = (int)aa.size()-1; d >= 0; d--)
		{
			q = (aa[d]-48)*(bb[c]-48)+carry;
			dgt[0] = (q % 10) + 48;
			tcum.insert(0,dgt);
			carry = q / 10;
		}
		if (carry > 0)
		{
			dgt[0] = carry + 48;
			tcum.insert(0,dgt);
		}
		tcum += placeShift;
		product = th_add(tcum, product);
		placeShift += (string)"0";
		carry = 0;
	}

	if (nz > 0)
	{
		q = (int)product.size() - nz;
		if (q < 0)
		{
			q *= -1;
			t = (string)"0";
			while ((int)t.size() < q) { t += t; }
			product.insert(0, t.substr(0,q));
			product.insert(0, (string)".");
		}
		else
		{
			product.insert(q, (string)".");
		}
	}

	return sign + th_trim(product);
}

string th_divide(string a, string b, bool intDivide)
{
	string aa = a;
	string bb = b;
	string sign;
	string signa;
	string signb;
	int o1, o2;
	int c, q;
	string t;
	string quotient;
	string divRegister;
	if (aa[0] == '-') { signa = (string)"-"; aa.erase(0,1); }
	if (bb[0] == '-') { signb = (string)"-"; bb.erase(0,1); }
	if (aa[0] == '+') { aa.erase(0,1); }
	if (bb[0] == '+') { bb.erase(0,1); }

	if ( signa == signb ) { sign = (string)""; }
	else { sign = (string)"-"; }

	if ( (aa == (string)"NAN") || (bb == (string)"NAN") ) { return (string)"NAN"; }
	else if (aa == (string)"INF") { if (bb == (string)"INF") { return (string)"NAN"; } else { return sign + (string)"INF"; } }
	else if (bb == (string)"INF") { return (string)"0"; }
	else if (th_zero(aa)) { if (th_zero(bb)) { return (string)"NAN"; } else { return (string)"0"; } }
	else if (th_zero(bb)) { return sign + (string)"INF"; }
	else if (th_equal(bb, (string)"1")) { return sign + aa; }

	if ( (int)aa.find('.')>=0 || (int)aa.find(',')>=0 || (int)bb.find('.')>=0 || (int)bb.find(',')>=0 )
	{
		aa = (string)"0" + th_trim(aa);
		bb = (string)"0" + th_trim(bb);
		o1 = (int)aa.find('.');
		if (o1 < 0) { o1 = (int)aa.find(','); }
		o2 = (int)bb.find('.');
		if (o2 < 0) { o2 = (int)bb.find(','); }
		while ( (o1 >= 0) || (o2 >= 0) )
		{
			if (o1 >= 0)
			{
				aa[o1] = aa[o1+1];
				aa[o1+1] = '.';
				if (o1+1 == ((int)aa.size()-1)) { aa.erase(o1+1,1); o1 = -1; }
				else { o1++; }
			}
			else { aa += (string)"0"; }
			if (o2 >= 0)
			{
				bb[o2] = bb[o2+1];
				bb[o2+1] = '.';
				if (o2+1 == ((int)bb.size()-1)) { bb.erase(o2+1,1); o2 = -1; }
				else { o2++; }
			}
			else { bb += (string)"0"; }
		}
	}

	aa = th_trim(aa);
	bb = th_trim(bb);
	quotient = (string)"";
	divRegister = (string)"";
	for (c = 0; c < (int)aa.size(); c++)
	{
		t = (string)" ";
		t[0] = aa[c];
		divRegister += t;
		t = th_fastdivide(divRegister, bb);
		quotient += t;
		t = th_multiply(t, bb);
		divRegister = th_subtract(divRegister, t);
	}
	th_lastquotient = quotient;
	if ( (!intDivide) && (! th_zero(divRegister)) )
	{
		quotient += (string)".";
		q = (int)quotient.size();
		while (!th_zero(divRegister))
		{
			divRegister += (string)"0";
			t = th_fastdivide(divRegister, bb);
			quotient += t;
			t = th_multiply(t, bb);
			divRegister = th_subtract(divRegister, t);
			if ( ((int)quotient.size() - q) >= th_precision ) { break; }
		}
	}

	return sign + th_trim(quotient);
}

string th_divide(string a, string b)
{
	return th_divide(a, b, false);
}

string th_div(string a, string b)
{
	return th_divide(a, b, true);
}

string th_mod(string a, string b)
{
	string q;
	string r;
	q = th_divide(a, b, true);
	r = th_multiply(q, b);
	return th_subtract(a, r);
}

string th_pow(string a, string b)
{
	string aa = a;
	string bb = b;
	string sign;
	string signa;
	string signb;
	string c;
	string p;
	if (aa[0] == '-') { signa = (string)"-"; aa.erase(0,1); }
	if (bb[0] == '-') { signb = (string)"-"; bb.erase(0,1); }
	if (aa[0] == '+') { aa.erase(0,1); }
	if (bb[0] == '+') { bb.erase(0,1); }

	if ( (aa == (string)"NAN") || (bb == (string)"NAN") ) { return (string)"NAN"; }
	else if (aa == (string)"INF")
	{
		if (th_zero(bb)) { return (string)"1"; }
		else if (signb == (string)"-") { return (string)"0"; }
		else if ( (bb == (string)"INF") && (signa != signb) ) { return (string)"NAN"; }
		else { return (string)"INF"; }
	}
	else if (th_zero(bb)) { return (string)"1"; }
	else if (bb == (string)"INF")
	{
		if (th_equal(aa, (string)"1")) { return (string)"1"; }
		else if ( (signb == (string)"") && (signa == (string)"-") ) { return (string)"NAN"; }
		else if ( (signb == (string)"-") && (signa == (string)"-") ) { return (string)"0"; }
		else if ( (signb == (string)"") && (th_compare(aa, (string)"1") == (string)"-1") ) { return (string)"0"; }
		else if ( (signb == (string)"-") && (th_compare(aa, (string)"1") == (string)"-1") ) { return (string)"INF"; }
		else if ( (signb == (string)"-") ) { return (string)"0"; }
		else { return (string)"INF"; }
	}
	else if ( signb == (string)"-" ) { return th_divide((string)"1",th_pow(a, bb)); }
	else if ( th_equal(a, (string)"1") ) { return (string)"1"; }
	else if ( th_equal(bb, (string)"1") ) { return th_trim(a); }
	else if ( th_equal(bb, (string)"0.5") ) { return th_sqrt(a); }
	else if ( th_equal(bb, (string)"1.5") ) { return th_multiply(a, th_sqrt(a)); }
	else if ( th_compare(bb, (string)"2") == (string)"-1" )
	{
		/* for powers 0 - 2 we use logarithmic functions */
		return th_exp(th_multiply(th_ln(aa),bb));
	}
	else if ( th_equal(th_trunc(bb),bb) )
	{
		/* for positive integer powers we use binary recursion */
		p = th_multiply(bb, (string)"0.5");
		c = th_pow(a, th_trunc(p));
		c = th_multiply(c,c);
		if ( (int)p.find('.') >= 0 ) { return th_multiply(a, c); }
		else { return c; }
	}
	else
	{
		/* for noninteger powers > 2 we use linear recursion */
		p = th_subtract(bb, (string)"1");
		c = th_pow(a, p);
		return th_multiply(a, c);
	}
}

string th_fastdivide(string a, string b)
{
	/*
		This should only be called from divide.
		It relies on the quotient being from 0-9.
		It does not do any range or sign checking.
		DO NOT CALL DIRECTLY.
	*/

	int c;
	string cc = (string)"1";
	if ( th_compare(a,b) == (string)"-1" ) { return (string)"0"; }
	if ( th_equal(a,b) ) { return (string)"1"; }
	for (c = 2; c < 11; c++)
	{
		cc[0]++; if (cc[0] == ':') { cc = (string)"10"; }
		if (th_compare(th_multiply(b,cc),a) == (string)"1")
		{
			if (cc[0] == '1') { return (string)"9"; }
			else { cc[0]--; return cc; }
		}
	}
	return (string)"0";
}

string th_lastmod(string a, string b)
{
	return th_subtract(a,th_multiply(th_lastquotient,b));
}

string th_abs(string a)
{
	string aa = a;
	if (aa[0] == '-') { aa.erase(0,1); }
	if (aa[0] == '+') { aa.erase(0,1); }
	return th_trim(aa);
}

string th_sign(string a)
{
	string aa = a;
	if (th_zero(aa)) { return (string)"0"; }
	else if (aa[0] == '-') { return (string)"-1"; }
	else { return (string)"1"; }
}

string th_trunc(string a)
{
	string aa = a;
	string signa;
	int o;
	if (aa[0] == '-') { signa = (string)"-"; aa.erase(0,1); }
	if (aa[0] == '+') { aa.erase(0,1); }
	o = (int)aa.find('.');
	if (o < 0) { o = (int)aa.find(','); }
	if (o >= 0) { aa.erase(o,(int)aa.size()-o); }
	return signa + th_trim(aa);
}

string th_square(string a)
{
	return th_multiply(a,a);
}

string th_sqrt(string a)
{
	string aa = a;
	string signa;
	int o, x, i, j, m;
	string dgt = (string)" ";
	string t;
	string r;
	string tr;
	string sqrtRegister;
	if (aa[0] == '-') { signa = (string)"-"; aa.erase(0,1); }
	if (aa[0] == '+') { aa.erase(0,1); }

	if ( (signa == (string)"-") || (aa == (string)"NAN") ) { return (string)"NAN"; }
	else if ( aa == (string)"INF" )
	{
		/*
			There was a young man from Trinity
			Who took the square root of Infinity.
			But the number of digits
			Gave him the fidgets.
			He dropped math and took up Divinity.
		*/
		return (string)"INF";
	}
	else if ( th_zero(aa) ) { return (string)"0"; }
	else if ( th_equal(aa, (string)"1") ) { return (string)"1"; }
	else if ( th_equal(aa, (string)"2") && (th_precision < th_const_prec) ) { return th__sqrt2().substr(0,th_precision+2); }
	else if ( th_equal(aa, (string)"3") && (th_precision < th_const_prec) ) { return th__sqrt3().substr(0,th_precision+2); }
	else if ( th_equal(aa, (string)"4") ) { return (string)"2"; }
	else if ( th_equal(aa, (string)"5") && (th_precision < th_const_prec) ) { return th__sqrt5().substr(0,th_precision+2); }
	else if ( th_equal(aa, (string)"6") && (th_precision < th_const_prec) ) { return th__sqrt6().substr(0,th_precision+2); }
	else if ( th_equal(aa, (string)"7") && (th_precision < th_const_prec) ) { return th__sqrt7().substr(0,th_precision+2); }
	else if ( th_equal(aa, (string)"8") && (th_precision < th_const_prec) ) { return th__sqrt8().substr(0,th_precision+2); }
	else if ( th_equal(aa, (string)"9") ) { return (string)"3"; }
	else if ( th_equal(aa, (string)"10") && (th_precision < th_const_prec) ) { return th__sqrt10().substr(0,th_precision+2); }

	aa = th_trim(aa);
	o = (int)aa.find('.');
	if (o < 0) { o = (int)aa.find(','); }
	if (o < 0) { o = (int)aa.size(); }
	else { aa.erase(o,1); }
	if ((o % 2) == 1) { aa.insert(0, (string)"0"); o++; }
	if (((int)aa.size() % 2) == 1) { aa += (string)"0"; }
	x = (2 * th_precision) - ((int)aa.size() - o);
	t = (string)"00";
	while ((int)t.size() < x) { t += t; }
	aa += t.substr(0,x);
	o /= 2;
	m = ((int)aa.size() / 2);
	r = (string)"";
	sqrtRegister = (string)"";
	for (i = 0; i < m; i++)
	{
		sqrtRegister += aa.substr(i*2, 2);
		tr = th_add(r, r);
		for (j = 0; j < 11; j++)
		{
			dgt[0] = j + 48;
			if (th_compare(th_multiply(tr + dgt, dgt), sqrtRegister) == (string)"1")
			{
				dgt[0]--;
				break;
			}
		}
		r += dgt;
		sqrtRegister = th_subtract(sqrtRegister, th_multiply(tr + dgt, dgt));
	}
	r.insert(o, (string)".");
	return th_trim(r);
}

string th_trim(string a)
{
	string aa = a;
	string signa;
	int m;
	if (aa[0] == '-') { signa = (string)"-"; aa.erase(0,1); }
	if (aa[0] == '+') { aa.erase(0,1); }

	while ( (aa[0] == '0') || (aa[0] == ' ') ) { aa.erase(0,1); }
	if ( ((int)aa.find('.') >= 0) || ((int)aa.find(',') >= 0) )
	{
		if ( (aa[0] == '.') || (aa[0] == ',') ) { aa.insert(0, (string)"0"); }
		m = (int)aa.size()-1;
		while ( (aa[m] == '0') || (aa[m] == ' ') ) { aa.erase(m, 1); m--; }
		if ( (aa[m] == '.') || (aa[m] == ',') ) { aa.erase(m, 1); m--; }
	}
	if ((int)aa.size() == 0) { return (string)"0"; }
	else if (((int)aa.size() == 1) && (aa[0] == '0')) { return (string)"0"; }
	else { return signa + aa; }
}

string th_maclaurin(string x, int indexMultiplier, int indexShift, bool denomFactorial,
			bool alternateSigns, bool startNegative, int indexStart, int indexStop)
{
	/*
		This will evaluate the following:

		indexStop-1
		-----
		 \       (indexMultiplier*k + indexShift)[    (k [+1]<if startNegative)]
		   \    x                                [(-1)                         ]< if alternateSigns
		   /    -----------------------------------------------------------------------------------
		 /           (indexMultiplier*k + indexShift) [!]<if denomFactorial
		-----
		k=indexStart

		Here's a table of values for the functions Thmath supports.
		x is determined from how accurate an answer we want.

		Function Mult Shft Factrl AltSgn StNeg  IdxS IdxE
		  sin(): 2,   1,   true,  true,  false, 0,   x
		  cos(): 2,   0,   true,  true,  false, 0,   x
		 atan(): 2,   1,   false, true,  false, 0,   x
		 sinh(): 2,   1,   true,  false, false, 0,   x
		 cosh(): 2,   0,   true,  false, false, 0,   x
		atanh(): 2,   1,   false, false, false, 0,   x
		  exp(): 1,   0,   true,  false, false, 0,   x

		ln() uses its own function because its Maclaurin series is different.
		Other functions are based off these because their Maclaurin series
		are quite messy and complicated.
	*/

	string a = (string)"0";
	int k;
	int e;
	string t;
	for (k = indexStart; k < indexStop; k++)
	{
		e = (indexMultiplier * k) + indexShift;
		if (denomFactorial && (e >= th_max_factorial)) { break; }
		t = th_divide(th_pow(x,th__str(e)),(denomFactorial?th_factorialtable(e):th__str(e)));
		if (th_zero(t)) { break; }
		if (alternateSigns && ((k % 2) == (startNegative?0:1))) { a = th_subtract(a, t); }
		else { a = th_add(a, t); }
	}
	return a;
}

string th_ln(string n)
{
	/*
		The Maclaurin series only converges for 0 < n  2.
		I get higher values by adding ln(2) to ln(n/2).
	*/

	string x = n;
	string a = (string)"0";
	int k;
	string t;
	string numer;
	string denom;
	if ((int)x.size() == 0) { return (string)"-INF"; }
	if (x[0] == '-') { return (string)"NAN"; }
	if (x[0] == '+') { x.erase(0,1); }
	if (th_zero(x)) { return (string)"-INF"; }
	else if (x == (string)"NAN") { return (string)"NAN"; }
	else if (x == (string)"INF") { return (string)"INF"; }
	else if (th_equal(x, (string)"0.5")) { return (string)"-" + th__ln2().substr(0,th_precision+2); }
	else if (th_equal(x, (string)"1")) { return (string)"0"; }
	else if (th_equal(x, (string)"2")) { return th__ln2().substr(0,th_precision+2); }
	else if (th_compare(x, (string)"2") == (string)"1")
	{
		return th_add(th__ln2().substr(0,th_precision+2), th_ln(th_multiply(x, (string)"0.5")));
	}
	else
	{
		for (k = 1; k < th_maclaur_limit; k++)
		{
			denom = th__str(k);
			numer = th_pow(th_subtract(x, (string)"1"),denom);
			t = th_divide(numer, denom);
			if (th_zero(t)) { break; }
			if ((k % 2) == 0) { a = th_subtract(a, t); }
			else { a = th_add(a, t); }
		}
		return a;
	}
}

string th_exp(string a)
{
	if ( (a == (string)"NAN") || (a == (string)"-NAN") ) { return (string)"NAN"; }
	else if (th_zero(a)) { return (string)"1"; }
	else if ( a == (string)"INF" ) { return (string)"INF"; }
	else if ( a == (string)"-INF" ) { return (string)"0"; }
	else if ( (th_equal(a,(string)"1")) ) { return th__e().substr(0,th_precision+2); }
	else { return th_maclaurin(a, 1, 0, true, false, false, 0, th_max_factorial); }
}

string th_sin(string a)
{
	if ( (a == (string)"NAN") || (a == (string)"-NAN") ) { return (string)"NAN"; }
	else if (th_zero(a)) { return (string)"0"; }
	else if ( a == (string)"INF" ) { return (string)"NAN"; }
	else if ( a == (string)"-INF" ) { return (string)"NAN"; }
	else { return th_maclaurin(a, 2, 1, true, true, false, 0, th_max_factorial); }
}

string th_cos(string a)
{
	if ( (a == (string)"NAN") || (a == (string)"-NAN") ) { return (string)"NAN"; }
	else if (th_zero(a)) { return (string)"1"; }
	else if ( a == (string)"INF" ) { return (string)"NAN"; }
	else if ( a == (string)"-INF" ) { return (string)"NAN"; }
	else { return th_maclaurin(a, 2, 0, true, true, false, 0, th_max_factorial); }
}

string th_atan(string a)
{
	/*
		The Maclaurin series only converges for -1  n  1.
		For larger numbers I use the formula
			atan(x) = 2*atan(x/2) - atan(x^3 / (4 + 3*x^2))
		derived from
			atan(1/x) = 2*atan(1/2x) - atan(1 / (4*x^3 + 3x))
		found at http://mathworld.wolfram.com/InverseTangent.html
	*/
	if ( (a == (string)"NAN") || (a == (string)"-NAN") ) { return (string)"NAN"; }
	else if (th_zero(a)) { return (string)"0"; }
	else if ( a == (string)"INF" ) { return th__pi2().substr(0,th_precision+2); }
	else if ( a == (string)"-INF" ) { return (string)"-" + th__pi2().substr(0,th_precision+2); }
	else if (th_equal(a, (string)"1")) { return th__pi4().substr(0,th_precision+2); }
	else if (th_equal(a, (string)"-1")) { return (string)"-" + th__pi4().substr(0,th_precision+2); }
	else if (th_compare(th_abs(a), (string)"1") == (string)"1")
	{
		return th_subtract(
			th_multiply(th_atan(th_multiply(a,(string)"0.5")),(string)"2"),
			th_atan(th_divide(th_multiply(a,th_square(a)),th_add(th_multiply(th_square(a),(string)"3"),(string)"4")))
		);
	}
	else { return th_maclaurin(a, 2, 1, false, true, false, 0, th_maclaur_limit); }
}

string th_sinh(string a)
{
	if ( (a == (string)"NAN") || (a == (string)"-NAN") ) { return (string)"NAN"; }
	else if (th_zero(a)) { return (string)"0"; }
	else if ( a == (string)"INF" ) { return (string)"INF"; }
	else if ( a == (string)"-INF" ) { return (string)"-INF"; }
	else { return th_maclaurin(a, 2, 1, true, false, false, 0, th_max_factorial); }
}

string th_cosh(string a)
{
	if ( (a == (string)"NAN") || (a == (string)"-NAN") ) { return (string)"NAN"; }
	else if (th_zero(a)) { return (string)"1"; }
	else if ( a == (string)"INF" ) { return (string)"INF"; }
	else if ( a == (string)"-INF" ) { return (string)"INF"; }
	else { return th_maclaurin(a, 2, 0, true, false, false, 0, th_max_factorial); }
}

string th_atanh(string a)
{
	if ( (a == (string)"NAN") || (a == (string)"-NAN") ) { return (string)"NAN"; }
	else if (th_zero(a)) { return (string)"0"; }
	else if (th_equal(a, (string)"1")) { return (string)"INF"; }
	else if (th_equal(a, (string)"-1")) { return (string)"-INF"; }
	else if (th_compare(th_abs(a),(string)"1") == (string)"1") { return (string)"NAN"; }
	else { return th_maclaurin(a, 2, 1, false, false, false, 0, th_maclaur_limit); }
}

string th_tan(string a)
{
	return th_divide(th_sin(a),th_cos(a));
}

string th_cot(string a)
{
	return th_divide(th_cos(a),th_sin(a));
}

string th_sec(string a)
{
	return th_divide((string)"1",th_cos(a));
}

string th_csc(string a)
{
	return th_divide((string)"1",th_sin(a));
}

string th_tanh(string a)
{
	return th_divide(th_sinh(a),th_cosh(a));
}

string th_coth(string a)
{
	return th_divide(th_cosh(a),th_sinh(a));
}

string th_sech(string a)
{
	return th_divide((string)"1",th_cosh(a));
}

string th_csch(string a)
{
	return th_divide((string)"1",th_sinh(a));
}

string th_asin(string a)
{
	if ( (a == (string)"NAN") || (a == (string)"-NAN") ) { return (string)"NAN"; }
	else if (th_zero(a)) { return (string)"0"; }
	else if (a == (string)"INF" ) { return (string)"NAN"; }
	else if (a == (string)"-INF") { return (string)"NAN"; }
	else if (th_equal(a,(string)"1") ) { return th__pi2().substr(0,th_precision+2); }
	else if (th_equal(a,(string)"-1")) { return (string)"-" + th__pi2().substr(0,th_precision+2); }
	else if (th_compare(th_abs(a),(string)"1") == (string)"1") { return (string)"NAN"; }
	else { return th_atan(th_divide(a,th_sqrt(th_subtract((string)"1",th_square(a))))); }
}

string th_acos(string a)
{
	if ( (a == (string)"NAN") || (a == (string)"-NAN") ) { return (string)"NAN"; }
	else if (th_zero(a)) { return th__pi2().substr(0,th_precision+2); }
	else if (a == (string)"INF" ) { return (string)"NAN"; }
	else if (a == (string)"-INF") { return (string)"NAN"; }
	else if (th_equal(a,(string)"1") ) { return (string)"0"; }
	else if (th_equal(a,(string)"-1")) { return th__pi().substr(0,th_precision+2); }
	else if (th_compare(th_abs(a),(string)"1") == (string)"1") { return (string)"NAN"; }
	else { return th_atan(th_divide(th_sqrt(th_subtract((string)"1",th_square(a))),a)); }
}

string th_asinh(string a)
{
	if ( (a == (string)"NAN") || (a == (string)"-NAN") ) { return (string)"NAN"; }
	else if (th_zero(a)) { return (string)"0"; }
	else if (a == (string)"INF" ) { return (string)"INF"; }
	else if (a == (string)"-INF") { return (string)"-INF"; }
	else { return th_ln(th_add(a,th_sqrt(th_add(th_square(a),(string)"1")))); }
}

string th_acosh(string a)
{
	if ( (a == (string)"NAN") || (a == (string)"-NAN") ) { return (string)"NAN"; }
	else if (a == (string)"INF" ) { return (string)"INF"; }
	else if (a == (string)"-INF") { return (string)"NAN"; }
	else if (th_equal(a,(string)"1") ) { return (string)"0"; }
	else if (th_compare(a,(string)"1") == (string)"-1") { return (string)"NAN"; }
	else { return th_ln(th_add(a,th_sqrt(th_subtract(th_square(a),(string)"1")))); }
}

string th_acot(string a)
{
	return th_atan(th_divide((string)"1",a));
}

string th_asec(string a)
{
	return th_acos(th_divide((string)"1",a));
}

string th_acsc(string a)
{
	return th_asin(th_divide((string)"1",a));
}

string th_acoth(string a)
{
	return th_atanh(th_divide((string)"1",a));
}

string th_asech(string a)
{
	return th_acosh(th_divide((string)"1",a));
}

string th_acsch(string a)
{
	return th_asinh(th_divide((string)"1",a));
}

string th_factorial(string a)
{
	string aa = th_trim(a);
	int n, i;
	if (aa == (string)"NAN") { return (string)"NAN"; }
	else if (aa == (string)"INF") { return (string)"INF"; }
	else if (aa == (string)"-INF") { return (string)"NAN"; }
	else if ( (th_equal(aa, (string)"-0.5")) ) { return th__sqrtpi().substr(0,th_precision+2); }
	else if ( (th_equal(aa, (string)"0.5")) ) { return th_multiply(th__sqrtpi(), (string)"0.5").substr(0,th_precision+2); }
	else if (th_equal(th_trunc(aa),aa))
	{
		if (aa[0] == '-')
		{
			return (string)"NAN";
		}
		else if (th_compare(aa, th__str(th_max_factorial)) == (string)"-1")
		{
			n = 0;
			for (i=0; i<(int)aa.size(); i++)
			{
				n = (n*10) + (aa[i]-48);
			}
			return th_factorialtable(n);
		}
		else
		{
			return th_multiply(th_factorial(th_subtract(a, (string)"1")), a);
		}
	}
	else
	{
		return th_gamma(th_add(a, (string)"1"));
	}
}

string th_gamma(string a)
{
	string aa = th_trim(a);
	if (aa == (string)"NAN") { return (string)"NAN"; }
	else if (aa == (string)"INF") { return (string)"INF"; }
	else if (aa == (string)"-INF") { return (string)"NAN"; }
	else if (th_equal(aa, (string)"0.5")) { return th__sqrtpi().substr(0,th_precision+2); }
	else if (th_equal(aa, (string)"1.5")) { return th_multiply(th__sqrtpi(), (string)"0.5").substr(0,th_precision+2); }
	else if (th_equal(th_trunc(aa),aa)) { return th_factorial(th_subtract(a, (string)"1")); }
	else if (aa[0] == '-')
	{
		return th_divide(th_gamma(th_add(aa, (string)"1")), aa);
	}
	else if (th_compare(aa, (string)"2") == (string)"1")
	{
		aa = th_subtract(aa, (string)"1");
		return th_multiply(th_gamma(aa), aa);
	}
	else if (th_compare(aa, (string)"1") == (string)"-1")
	{
		return th_divide(th_gamma(th_add(aa, (string)"1")), aa);
	}
	else /* we're left with an input between 1 and 2 */
	{
		/*
			I use the formula

				             (zz - 0.5) * ln(zz + 4.5) - zz - 4.5
				gamma(zz)  e                                     * sqrt(2) *

				[     76.180092   86.50532   24.014098   1.2317395   0.00120858   0.00000536 ]
				[ 1 + --------- - -------- + --------- - --------- + ---------- - ---------- ] for real(zz)  1
				[         zz        zz+1        zz+2        zz+3        zz+4         zz+5    ]

			from Calculus Tools, Concept Version 1.00,  8/7/2001 Texas Instruments.
		*/
		return th_multiply(
			th_exp(th_subtract(th_multiply(th_subtract(a, (string)"0.5"), th_ln(th_add(a, (string)"4.5"))), th_add(a,(string)"4.5"))),
			th_multiply(
				th_multiply(th__sqrtpi(),th__sqrt2()),
				th_subtract(
					th_add(th_add((string)"1",th_divide((string)"76.180092",a)),th_add(th_divide((string)"24.014098",
						th_add(a,(string)"2")),th_divide((string)"0.00120858",th_add(a,(string)"4")))),
					th_add(th_add(th_divide((string)"86.50532",th_add(a,(string)"1")),th_divide((string)"1.2317895",
						th_add(a,(string)"3"))),th_divide((string)"0.00000536",th_add(a,(string)"5")))
				)
			)
		);
	}
}

string th_ccompare(string x, string y)
{
	string a, b, c, d;
	string rx, ry;
	int xo = (int)x.find(';');
	int yo = (int)y.find(';');
	if (xo < 0) { a = x; b = (string)"0"; }
	else { a = x.substr(0,xo); b = x.substr(xo+1, (int)x.size()-xo-1); }
	if (yo < 0) { c = y; d = (string)"0"; }
	else { c = y.substr(0,yo); d = y.substr(yo+1, (int)y.size()-yo-1); }

	rx = th_compare(a, c);
	ry = th_compare(b, d);

	if (th_zero(ry)) { return rx; }
	else { return rx + (string)";" + ry; }
}

string th_cadd(string x, string y)
{
	string a, b, c, d;
	string rx, ry;
	int xo = (int)x.find(';');
	int yo = (int)y.find(';');
	if (xo < 0) { a = x; b = (string)"0"; }
	else { a = x.substr(0,xo); b = x.substr(xo+1, (int)x.size()-xo-1); }
	if (yo < 0) { c = y; d = (string)"0"; }
	else { c = y.substr(0,yo); d = y.substr(yo+1, (int)y.size()-yo-1); }

	rx = th_add(a, c);
	ry = th_add(b, d);

	if (th_zero(ry)) { return rx; }
	else { return rx + (string)";" + ry; }
}

string th_csubtract(string x, string y)
{
	string a, b, c, d;
	string rx, ry;
	int xo = (int)x.find(';');
	int yo = (int)y.find(';');
	if (xo < 0) { a = x; b = (string)"0"; }
	else { a = x.substr(0,xo); b = x.substr(xo+1, (int)x.size()-xo-1); }
	if (yo < 0) { c = y; d = (string)"0"; }
	else { c = y.substr(0,yo); d = y.substr(yo+1, (int)y.size()-yo-1); }

	rx = th_subtract(a, c);
	ry = th_subtract(b, d);

	if (th_zero(ry)) { return rx; }
	else { return rx + (string)";" + ry; }
}

string th_cmultiply(string x, string y)
{
	string a, b, c, d;
	string rx, ry;
	int xo = (int)x.find(';');
	int yo = (int)y.find(';');
	if (xo < 0) { a = x; b = (string)"0"; }
	else { a = x.substr(0,xo); b = x.substr(xo+1, (int)x.size()-xo-1); }
	if (yo < 0) { c = y; d = (string)"0"; }
	else { c = y.substr(0,yo); d = y.substr(yo+1, (int)y.size()-yo-1); }

	rx = th_subtract( th_multiply(a, c), th_multiply(b, d) );
	ry =      th_add( th_multiply(a, d), th_multiply(b, c) );

	if (th_zero(ry)) { return rx; }
	else { return rx + (string)";" + ry; }
}

string th_cdivide(string x, string y, bool idv)
{
	string a, b, c, d;
	string rx, ry;
	string p;
	int xo = (int)x.find(';');
	int yo = (int)y.find(';');
	if (xo < 0) { a = x; b = (string)"0"; }
	else { a = x.substr(0,xo); b = x.substr(xo+1, (int)x.size()-xo-1); }
	if (yo < 0) { c = y; d = (string)"0"; }
	else { c = y.substr(0,yo); d = y.substr(yo+1, (int)y.size()-yo-1); }

	p  =    th_add( th_square(c), th_square(d) );
	rx = th_divide(      th_add( th_multiply(a, c), th_multiply(b, d) ), p, idv );
	ry = th_divide( th_subtract( th_multiply(b, c), th_multiply(a, d) ), p, idv );

	if (th_zero(ry)) { return rx; }
	else { return rx + (string)";" + ry; }
}

string th_cdivide(string x, string y)
{
	return th_cdivide(x, y, false);
}

string th_cdiv(string x, string y)
{
	return th_cdivide(x, y, true);
}

string th_cmod(string x, string y)
{
	string a, b, c, d;
	string rx, ry;
	string p;
	int xo = (int)x.find(';');
	int yo = (int)y.find(';');
	if (xo < 0) { a = x; b = (string)"0"; }
	else { a = x.substr(0,xo); b = x.substr(xo+1, (int)x.size()-xo-1); }
	if (yo < 0) { c = y; d = (string)"0"; }
	else { c = y.substr(0,yo); d = y.substr(yo+1, (int)y.size()-yo-1); }

	p  = th_add( th_square(c), th_square(d) );
	rx = th_mod(      th_add( th_multiply(a, c), th_multiply(b, d) ), p );
	ry = th_mod( th_subtract( th_multiply(b, c), th_multiply(a, d) ), p );

	if (th_zero(ry)) { return rx; }
	else { return rx + (string)";" + ry; }
}

string th_cpow(string x, string y)
{
	string a, b, c, d;
	string rx, ry;
	string p, q, r, s;
	int xo = (int)x.find(';');
	int yo = (int)y.find(';');
	if (xo < 0) { a = x; b = (string)"0"; }
	else { a = x.substr(0,xo); b = x.substr(xo+1, (int)x.size()-xo-1); }
	if (yo < 0) { c = y; d = (string)"0"; }
	else { c = y.substr(0,yo); d = y.substr(yo+1, (int)y.size()-yo-1); }

	p  = th_subtract(th_atan(th_divide(a,b)), th_multiply(th__pi2(), th_sign(b)));
	q  = th_add(th_square(a), th_square(b));
	r  = th_subtract(th_multiply(th_multiply(d, th_ln(q)), (string)"0.5"), th_multiply(c, p));
	s  = th_multiply(th_pow(th_sqrt(q), c), th_exp(th_multiply(d,p)));
	rx = th_multiply(s, th_cos(r));
	ry = th_multiply(s, th_sin(r));

	if (th_zero(ry)) { return rx; }
	else { return rx + (string)";" + ry; }
}

string th_cabs(string x)
{
	string a, b;
	string rx, ry;
	int xo = (int)x.find(';');
	if (xo < 0) { a = x; b = (string)"0"; }
	else { a = x.substr(0,xo); b = x.substr(xo+1, (int)x.size()-xo-1); }

	rx = th_sqrt(th_add(th_square(a),th_square(b)));
	ry = (string)"0";

	if (th_zero(ry)) { return rx; }
	else { return rx + (string)";" + ry; }
}

string th_csquare(string x)
{
	string a, b;
	string rx, ry;
	int xo = (int)x.find(';');
	if (xo < 0) { a = x; b = (string)"0"; }
	else { a = x.substr(0,xo); b = x.substr(xo+1, (int)x.size()-xo-1); }

	rx = th_subtract(th_square(a), th_square(b));
	ry = th_multiply(th_multiply(a, b), (string)"2");

	if (th_zero(ry)) { return rx; }
	else { return rx + (string)";" + ry; }
}

string th_csqrt(string x)
{
	string a, b;
	string rx, ry;
	string p;
	int xo = (int)x.find(';');
	if (xo < 0) { a = x; b = (string)"0"; }
	else { a = x.substr(0,xo); b = x.substr(xo+1, (int)x.size()-xo-1); }

	if (th_zero(b))
	{
		if (th_sign(a) == (string)"-1") { return (string)"0;" + th_sqrt(th_abs(a)); }
		else { return th_sqrt(a); }
	}
	p = th_sqrt(th_add(th_square(a),th_square(b)));
	rx = th_multiply(th_multiply(th_sqrt(th_add(p,a)), th__sqrt2().substr(0, th_precision)), (string)"0.5");
	ry = th_multiply(th_multiply(th_sqrt(th_subtract(p,a)), th__sqrt2().substr(0, th_precision)), (string)"0.5");
	if (b[0] == '-') { ry.insert(0, (string)"-"); }

	if (th_zero(ry)) { return rx; }
	else { return rx + (string)";" + ry; }
}

bool th_czero(string x)
{
	string a, b;
	int xo = (int)x.find(';');
	if (xo < 0) { a = x; b = (string)"0"; }
	else { a = x.substr(0,xo); b = x.substr(xo+1, (int)x.size()-xo-1); }
	return ( th_zero(a) && th_zero(b) );
}

bool th_cequal(string x, string y)
{
	string a, b, c, d;
	int xo = (int)x.find(';');
	int yo = (int)y.find(';');
	if (xo < 0) { a = x; b = (string)"0"; }
	else { a = x.substr(0,xo); b = x.substr(xo+1, (int)x.size()-xo-1); }
	if (yo < 0) { c = y; d = (string)"0"; }
	else { c = y.substr(0,yo); d = y.substr(yo+1, (int)y.size()-yo-1); }
	return ( th_equal(a, c) && th_equal(b, d) );
}

string th_ctrunc(string x)
{
	string a, b;
	string rx, ry;
	int xo = (int)x.find(';');
	if (xo < 0) { a = x; b = (string)"0"; }
	else { a = x.substr(0,xo); b = x.substr(xo+1, (int)x.size()-xo-1); }

	rx = th_trunc(a);
	ry = th_trunc(b);

	if (th_zero(ry)) { return rx; }
	else { return rx + (string)";" + ry; }
}

string th_csign(string x)
{
	string a, b;
	string rx, ry;
	int xo = (int)x.find(';');
	if (xo < 0) { a = x; b = (string)"0"; }
	else { a = x.substr(0,xo); b = x.substr(xo+1, (int)x.size()-xo-1); }

	rx = th_sign(a);
	ry = th_sign(b);

	if (th_zero(ry)) { return rx; }
	else { return rx + (string)";" + ry; }
}

string th_ctrim(string x)
{
	string a, b;
	string rx, ry;
	int xo = (int)x.find(';');
	if (xo < 0) { a = x; b = (string)"0"; }
	else { a = x.substr(0,xo); b = x.substr(xo+1, (int)x.size()-xo-1); }

	rx = th_trim(a);
	ry = th_trim(b);

	if (th_zero(ry)) { return rx; }
	else { return rx + (string)";" + ry; }
}

/* FUNCTIONS USED INTERNALLY BY THMATH */

string th__str(int x)
{
	int xx = x;
	string s;
	string dgt = (string)" ";
	if (xx == 0) { return (string)"0"; }
	else if (xx < 0) { return (string)"-" + th__str(-xx); }
	else
	{
		while (xx > 0)
		{
			dgt[0] = (xx % 10) + 48;
			s.insert(0, dgt);
			xx /= 10;
		}
	}
	return s;
}

void th__test(int nlen, int mode, int c)
{
	string s1 = (string)"98765432109876543210987654321098765432109876543210987654321098765432109876543210";
	string s2 = (string)"12345678901234567890123456789012345678901234567890123456789012345678901234567890";
	s1.erase(nlen,80-nlen);
	s2.erase(nlen,80-nlen);
	string s;
	int i;
	for (i=0; i<c; i++)
	{
		switch (mode) {
		case 0:
			s = th_add(s1,s2);
			break;
		case 1:
			s = th_subtract(s1,s2);
			break;
		case 2:
			s = th_multiply(s1,s2);
			break;
		case 3:
			s = th_divide(s1,s2);
			break;
		case 4:
			s = th_pow(s1,(string)"5");
			break;
		case 5:
			s = th_sqrt(s1);
			break;
		case 6:
			s = th_ln(s1);
			break;
		case 7:
			s = th_sin(s1);
			break;
		case 8:
			s = th_cos(s1);
			break;
		case 9:
			s = th_atan(s1);
			break;
		case 10:
			s = th_tan(s1);
			break;
		}
	}
}



